@@ -6,6 +6,8 @@
 # pylint: disable=too-many-instance-attributes
 from enum import Enum
 
+from azure.core import CaseInsensitiveEnumMeta
+
 
 def get_enum_value(value):
     if value is None or value in ["None", ""]:
@@ -16,170 +18,174 @@ def get_enum_value(value):
         return value
 
 
-class StorageErrorCode(str, Enum):
+class StorageErrorCode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
 
     # Generic storage values
-    account_already_exists = "AccountAlreadyExists"
-    account_being_created = "AccountBeingCreated"
-    account_is_disabled = "AccountIsDisabled"
-    authentication_failed = "AuthenticationFailed"
-    authorization_failure = "AuthorizationFailure"
-    no_authentication_information = "NoAuthenticationInformation"
-    condition_headers_not_supported = "ConditionHeadersNotSupported"
-    condition_not_met = "ConditionNotMet"
-    empty_metadata_key = "EmptyMetadataKey"
-    insufficient_account_permissions = "InsufficientAccountPermissions"
-    internal_error = "InternalError"
-    invalid_authentication_info = "InvalidAuthenticationInfo"
-    invalid_header_value = "InvalidHeaderValue"
-    invalid_http_verb = "InvalidHttpVerb"
-    invalid_input = "InvalidInput"
-    invalid_md5 = "InvalidMd5"
-    invalid_metadata = "InvalidMetadata"
-    invalid_query_parameter_value = "InvalidQueryParameterValue"
-    invalid_range = "InvalidRange"
-    invalid_resource_name = "InvalidResourceName"
-    invalid_uri = "InvalidUri"
-    invalid_xml_document = "InvalidXmlDocument"
-    invalid_xml_node_value = "InvalidXmlNodeValue"
-    md5_mismatch = "Md5Mismatch"
-    metadata_too_large = "MetadataTooLarge"
-    missing_content_length_header = "MissingContentLengthHeader"
-    missing_required_query_parameter = "MissingRequiredQueryParameter"
-    missing_required_header = "MissingRequiredHeader"
-    missing_required_xml_node = "MissingRequiredXmlNode"
-    multiple_condition_headers_not_supported = "MultipleConditionHeadersNotSupported"
-    operation_timed_out = "OperationTimedOut"
-    out_of_range_input = "OutOfRangeInput"
-    out_of_range_query_parameter_value = "OutOfRangeQueryParameterValue"
-    request_body_too_large = "RequestBodyTooLarge"
-    resource_type_mismatch = "ResourceTypeMismatch"
-    request_url_failed_to_parse = "RequestUrlFailedToParse"
-    resource_already_exists = "ResourceAlreadyExists"
-    resource_not_found = "ResourceNotFound"
-    server_busy = "ServerBusy"
-    unsupported_header = "UnsupportedHeader"
-    unsupported_xml_node = "UnsupportedXmlNode"
-    unsupported_query_parameter = "UnsupportedQueryParameter"
-    unsupported_http_verb = "UnsupportedHttpVerb"
+    ACCOUNT_ALREADY_EXISTS = "AccountAlreadyExists"
+    ACCOUNT_BEING_CREATED = "AccountBeingCreated"
+    ACCOUNT_IS_DISABLED = "AccountIsDisabled"
+    AUTHENTICATION_FAILED = "AuthenticationFailed"
+    AUTHORIZATION_FAILURE = "AuthorizationFailure"
+    NO_AUTHENTICATION_INFORMATION = "NoAuthenticationInformation"
+    CONDITION_HEADERS_NOT_SUPPORTED = "ConditionHeadersNotSupported"
+    CONDITION_NOT_MET = "ConditionNotMet"
+    EMPTY_METADATA_KEY = "EmptyMetadataKey"
+    INSUFFICIENT_ACCOUNT_PERMISSIONS = "InsufficientAccountPermissions"
+    INTERNAL_ERROR = "InternalError"
+    INVALID_AUTHENTICATION_INFO = "InvalidAuthenticationInfo"
+    INVALID_HEADER_VALUE = "InvalidHeaderValue"
+    INVALID_HTTP_VERB = "InvalidHttpVerb"
+    INVALID_INPUT = "InvalidInput"
+    INVALID_MD5 = "InvalidMd5"
+    INVALID_METADATA = "InvalidMetadata"
+    INVALID_QUERY_PARAMETER_VALUE = "InvalidQueryParameterValue"
+    INVALID_RANGE = "InvalidRange"
+    INVALID_RESOURCE_NAME = "InvalidResourceName"
+    INVALID_URI = "InvalidUri"
+    INVALID_XML_DOCUMENT = "InvalidXmlDocument"
+    INVALID_XML_NODE_VALUE = "InvalidXmlNodeValue"
+    MD5_MISMATCH = "Md5Mismatch"
+    METADATA_TOO_LARGE = "MetadataTooLarge"
+    MISSING_CONTENT_LENGTH_HEADER = "MissingContentLengthHeader"
+    MISSING_REQUIRED_QUERY_PARAMETER = "MissingRequiredQueryParameter"
+    MISSING_REQUIRED_HEADER = "MissingRequiredHeader"
+    MISSING_REQUIRED_XML_NODE = "MissingRequiredXmlNode"
+    MULTIPLE_CONDITION_HEADERS_NOT_SUPPORTED = "MultipleConditionHeadersNotSupported"
+    OPERATION_TIMED_OUT = "OperationTimedOut"
+    OUT_OF_RANGE_INPUT = "OutOfRangeInput"
+    OUT_OF_RANGE_QUERY_PARAMETER_VALUE = "OutOfRangeQueryParameterValue"
+    REQUEST_BODY_TOO_LARGE = "RequestBodyTooLarge"
+    RESOURCE_TYPE_MISMATCH = "ResourceTypeMismatch"
+    REQUEST_URL_FAILED_TO_PARSE = "RequestUrlFailedToParse"
+    RESOURCE_ALREADY_EXISTS = "ResourceAlreadyExists"
+    RESOURCE_NOT_FOUND = "ResourceNotFound"
+    SERVER_BUSY = "ServerBusy"
+    UNSUPPORTED_HEADER = "UnsupportedHeader"
+    UNSUPPORTED_XML_NODE = "UnsupportedXmlNode"
+    UNSUPPORTED_QUERY_PARAMETER = "UnsupportedQueryParameter"
+    UNSUPPORTED_HTTP_VERB = "UnsupportedHttpVerb"
 
     # Blob values
-    append_position_condition_not_met = "AppendPositionConditionNotMet"
-    blob_already_exists = "BlobAlreadyExists"
-    blob_not_found = "BlobNotFound"
-    blob_overwritten = "BlobOverwritten"
-    blob_tier_inadequate_for_content_length = "BlobTierInadequateForContentLength"
-    block_count_exceeds_limit = "BlockCountExceedsLimit"
-    block_list_too_long = "BlockListTooLong"
-    cannot_change_to_lower_tier = "CannotChangeToLowerTier"
-    cannot_verify_copy_source = "CannotVerifyCopySource"
-    container_already_exists = "ContainerAlreadyExists"
-    container_being_deleted = "ContainerBeingDeleted"
-    container_disabled = "ContainerDisabled"
-    container_not_found = "ContainerNotFound"
-    content_length_larger_than_tier_limit = "ContentLengthLargerThanTierLimit"
-    copy_across_accounts_not_supported = "CopyAcrossAccountsNotSupported"
-    copy_id_mismatch = "CopyIdMismatch"
-    feature_version_mismatch = "FeatureVersionMismatch"
-    incremental_copy_blob_mismatch = "IncrementalCopyBlobMismatch"
-    incremental_copy_of_eralier_version_snapshot_not_allowed = "IncrementalCopyOfEralierVersionSnapshotNotAllowed"
-    incremental_copy_source_must_be_snapshot = "IncrementalCopySourceMustBeSnapshot"
-    infinite_lease_duration_required = "InfiniteLeaseDurationRequired"
-    invalid_blob_or_block = "InvalidBlobOrBlock"
-    invalid_blob_tier = "InvalidBlobTier"
-    invalid_blob_type = "InvalidBlobType"
-    invalid_block_id = "InvalidBlockId"
-    invalid_block_list = "InvalidBlockList"
-    invalid_operation = "InvalidOperation"
-    invalid_page_range = "InvalidPageRange"
-    invalid_source_blob_type = "InvalidSourceBlobType"
-    invalid_source_blob_url = "InvalidSourceBlobUrl"
-    invalid_version_for_page_blob_operation = "InvalidVersionForPageBlobOperation"
-    lease_already_present = "LeaseAlreadyPresent"
-    lease_already_broken = "LeaseAlreadyBroken"
-    lease_id_mismatch_with_blob_operation = "LeaseIdMismatchWithBlobOperation"
-    lease_id_mismatch_with_container_operation = "LeaseIdMismatchWithContainerOperation"
-    lease_id_mismatch_with_lease_operation = "LeaseIdMismatchWithLeaseOperation"
-    lease_id_missing = "LeaseIdMissing"
-    lease_is_breaking_and_cannot_be_acquired = "LeaseIsBreakingAndCannotBeAcquired"
-    lease_is_breaking_and_cannot_be_changed = "LeaseIsBreakingAndCannotBeChanged"
-    lease_is_broken_and_cannot_be_renewed = "LeaseIsBrokenAndCannotBeRenewed"
-    lease_lost = "LeaseLost"
-    lease_not_present_with_blob_operation = "LeaseNotPresentWithBlobOperation"
-    lease_not_present_with_container_operation = "LeaseNotPresentWithContainerOperation"
-    lease_not_present_with_lease_operation = "LeaseNotPresentWithLeaseOperation"
-    max_blob_size_condition_not_met = "MaxBlobSizeConditionNotMet"
-    no_pending_copy_operation = "NoPendingCopyOperation"
-    operation_not_allowed_on_incremental_copy_blob = "OperationNotAllowedOnIncrementalCopyBlob"
-    pending_copy_operation = "PendingCopyOperation"
-    previous_snapshot_cannot_be_newer = "PreviousSnapshotCannotBeNewer"
-    previous_snapshot_not_found = "PreviousSnapshotNotFound"
-    previous_snapshot_operation_not_supported = "PreviousSnapshotOperationNotSupported"
-    sequence_number_condition_not_met = "SequenceNumberConditionNotMet"
-    sequence_number_increment_too_large = "SequenceNumberIncrementTooLarge"
-    snapshot_count_exceeded = "SnapshotCountExceeded"
-    snaphot_operation_rate_exceeded = "SnaphotOperationRateExceeded"
-    snapshots_present = "SnapshotsPresent"
-    source_condition_not_met = "SourceConditionNotMet"
-    system_in_use = "SystemInUse"
-    target_condition_not_met = "TargetConditionNotMet"
-    unauthorized_blob_overwrite = "UnauthorizedBlobOverwrite"
-    blob_being_rehydrated = "BlobBeingRehydrated"
-    blob_archived = "BlobArchived"
-    blob_not_archived = "BlobNotArchived"
+    APPEND_POSITION_CONDITION_NOT_MET = "AppendPositionConditionNotMet"
+    BLOB_ALREADY_EXISTS = "BlobAlreadyExists"
+    BLOB_NOT_FOUND = "BlobNotFound"
+    BLOB_OVERWRITTEN = "BlobOverwritten"
+    BLOB_TIER_INADEQUATE_FOR_CONTENT_LENGTH = "BlobTierInadequateForContentLength"
+    BLOCK_COUNT_EXCEEDS_LIMIT = "BlockCountExceedsLimit"
+    BLOCK_LIST_TOO_LONG = "BlockListTooLong"
+    CANNOT_CHANGE_TO_LOWER_TIER = "CannotChangeToLowerTier"
+    CANNOT_VERIFY_COPY_SOURCE = "CannotVerifyCopySource"
+    CONTAINER_ALREADY_EXISTS = "ContainerAlreadyExists"
+    CONTAINER_BEING_DELETED = "ContainerBeingDeleted"
+    CONTAINER_DISABLED = "ContainerDisabled"
+    CONTAINER_NOT_FOUND = "ContainerNotFound"
+    CONTENT_LENGTH_LARGER_THAN_TIER_LIMIT = "ContentLengthLargerThanTierLimit"
+    COPY_ACROSS_ACCOUNTS_NOT_SUPPORTED = "CopyAcrossAccountsNotSupported"
+    COPY_ID_MISMATCH = "CopyIdMismatch"
+    FEATURE_VERSION_MISMATCH = "FeatureVersionMismatch"
+    INCREMENTAL_COPY_BLOB_MISMATCH = "IncrementalCopyBlobMismatch"
+    INCREMENTAL_COPY_OF_EARLIER_VERSION_SNAPSHOT_NOT_ALLOWED = "IncrementalCopyOfEarlierVersionSnapshotNotAllowed"
+    #: Deprecated: Please use INCREMENTAL_COPY_OF_EARLIER_VERSION_SNAPSHOT_NOT_ALLOWED instead.
+    INCREMENTAL_COPY_OF_ERALIER_VERSION_SNAPSHOT_NOT_ALLOWED = "IncrementalCopyOfEarlierVersionSnapshotNotAllowed"
+    INCREMENTAL_COPY_SOURCE_MUST_BE_SNAPSHOT = "IncrementalCopySourceMustBeSnapshot"
+    INFINITE_LEASE_DURATION_REQUIRED = "InfiniteLeaseDurationRequired"
+    INVALID_BLOB_OR_BLOCK = "InvalidBlobOrBlock"
+    INVALID_BLOB_TIER = "InvalidBlobTier"
+    INVALID_BLOB_TYPE = "InvalidBlobType"
+    INVALID_BLOCK_ID = "InvalidBlockId"
+    INVALID_BLOCK_LIST = "InvalidBlockList"
+    INVALID_OPERATION = "InvalidOperation"
+    INVALID_PAGE_RANGE = "InvalidPageRange"
+    INVALID_SOURCE_BLOB_TYPE = "InvalidSourceBlobType"
+    INVALID_SOURCE_BLOB_URL = "InvalidSourceBlobUrl"
+    INVALID_VERSION_FOR_PAGE_BLOB_OPERATION = "InvalidVersionForPageBlobOperation"
+    LEASE_ALREADY_PRESENT = "LeaseAlreadyPresent"
+    LEASE_ALREADY_BROKEN = "LeaseAlreadyBroken"
+    LEASE_ID_MISMATCH_WITH_BLOB_OPERATION = "LeaseIdMismatchWithBlobOperation"
+    LEASE_ID_MISMATCH_WITH_CONTAINER_OPERATION = "LeaseIdMismatchWithContainerOperation"
+    LEASE_ID_MISMATCH_WITH_LEASE_OPERATION = "LeaseIdMismatchWithLeaseOperation"
+    LEASE_ID_MISSING = "LeaseIdMissing"
+    LEASE_IS_BREAKING_AND_CANNOT_BE_ACQUIRED = "LeaseIsBreakingAndCannotBeAcquired"
+    LEASE_IS_BREAKING_AND_CANNOT_BE_CHANGED = "LeaseIsBreakingAndCannotBeChanged"
+    LEASE_IS_BROKEN_AND_CANNOT_BE_RENEWED = "LeaseIsBrokenAndCannotBeRenewed"
+    LEASE_LOST = "LeaseLost"
+    LEASE_NOT_PRESENT_WITH_BLOB_OPERATION = "LeaseNotPresentWithBlobOperation"
+    LEASE_NOT_PRESENT_WITH_CONTAINER_OPERATION = "LeaseNotPresentWithContainerOperation"
+    LEASE_NOT_PRESENT_WITH_LEASE_OPERATION = "LeaseNotPresentWithLeaseOperation"
+    MAX_BLOB_SIZE_CONDITION_NOT_MET = "MaxBlobSizeConditionNotMet"
+    NO_PENDING_COPY_OPERATION = "NoPendingCopyOperation"
+    OPERATION_NOT_ALLOWED_ON_INCREMENTAL_COPY_BLOB = "OperationNotAllowedOnIncrementalCopyBlob"
+    PENDING_COPY_OPERATION = "PendingCopyOperation"
+    PREVIOUS_SNAPSHOT_CANNOT_BE_NEWER = "PreviousSnapshotCannotBeNewer"
+    PREVIOUS_SNAPSHOT_NOT_FOUND = "PreviousSnapshotNotFound"
+    PREVIOUS_SNAPSHOT_OPERATION_NOT_SUPPORTED = "PreviousSnapshotOperationNotSupported"
+    SEQUENCE_NUMBER_CONDITION_NOT_MET = "SequenceNumberConditionNotMet"
+    SEQUENCE_NUMBER_INCREMENT_TOO_LARGE = "SequenceNumberIncrementTooLarge"
+    SNAPSHOT_COUNT_EXCEEDED = "SnapshotCountExceeded"
+    SNAPSHOT_OPERATION_RATE_EXCEEDED = "SnapshotOperationRateExceeded"
+    #: Deprecated: Please use SNAPSHOT_OPERATION_RATE_EXCEEDED instead.
+    SNAPHOT_OPERATION_RATE_EXCEEDED = "SnapshotOperationRateExceeded"
+    SNAPSHOTS_PRESENT = "SnapshotsPresent"
+    SOURCE_CONDITION_NOT_MET = "SourceConditionNotMet"
+    SYSTEM_IN_USE = "SystemInUse"
+    TARGET_CONDITION_NOT_MET = "TargetConditionNotMet"
+    UNAUTHORIZED_BLOB_OVERWRITE = "UnauthorizedBlobOverwrite"
+    BLOB_BEING_REHYDRATED = "BlobBeingRehydrated"
+    BLOB_ARCHIVED = "BlobArchived"
+    BLOB_NOT_ARCHIVED = "BlobNotArchived"
 
     # Queue values
-    invalid_marker = "InvalidMarker"
-    message_not_found = "MessageNotFound"
-    message_too_large = "MessageTooLarge"
-    pop_receipt_mismatch = "PopReceiptMismatch"
-    queue_already_exists = "QueueAlreadyExists"
-    queue_being_deleted = "QueueBeingDeleted"
-    queue_disabled = "QueueDisabled"
-    queue_not_empty = "QueueNotEmpty"
-    queue_not_found = "QueueNotFound"
+    INVALID_MARKER = "InvalidMarker"
+    MESSAGE_NOT_FOUND = "MessageNotFound"
+    MESSAGE_TOO_LARGE = "MessageTooLarge"
+    POP_RECEIPT_MISMATCH = "PopReceiptMismatch"
+    QUEUE_ALREADY_EXISTS = "QueueAlreadyExists"
+    QUEUE_BEING_DELETED = "QueueBeingDeleted"
+    QUEUE_DISABLED = "QueueDisabled"
+    QUEUE_NOT_EMPTY = "QueueNotEmpty"
+    QUEUE_NOT_FOUND = "QueueNotFound"
 
     # File values
-    cannot_delete_file_or_directory = "CannotDeleteFileOrDirectory"
-    client_cache_flush_delay = "ClientCacheFlushDelay"
-    delete_pending = "DeletePending"
-    directory_not_empty = "DirectoryNotEmpty"
-    file_lock_conflict = "FileLockConflict"
-    invalid_file_or_directory_path_name = "InvalidFileOrDirectoryPathName"
-    parent_not_found = "ParentNotFound"
-    read_only_attribute = "ReadOnlyAttribute"
-    share_already_exists = "ShareAlreadyExists"
-    share_being_deleted = "ShareBeingDeleted"
-    share_disabled = "ShareDisabled"
-    share_not_found = "ShareNotFound"
-    sharing_violation = "SharingViolation"
-    share_snapshot_in_progress = "ShareSnapshotInProgress"
-    share_snapshot_count_exceeded = "ShareSnapshotCountExceeded"
-    share_snapshot_operation_not_supported = "ShareSnapshotOperationNotSupported"
-    share_has_snapshots = "ShareHasSnapshots"
-    container_quota_downgrade_not_allowed = "ContainerQuotaDowngradeNotAllowed"
+    CANNOT_DELETE_FILE_OR_DIRECTORY = "CannotDeleteFileOrDirectory"
+    CLIENT_CACHE_FLUSH_DELAY = "ClientCacheFlushDelay"
+    DELETE_PENDING = "DeletePending"
+    DIRECTORY_NOT_EMPTY = "DirectoryNotEmpty"
+    FILE_LOCK_CONFLICT = "FileLockConflict"
+    INVALID_FILE_OR_DIRECTORY_PATH_NAME = "InvalidFileOrDirectoryPathName"
+    PARENT_NOT_FOUND = "ParentNotFound"
+    READ_ONLY_ATTRIBUTE = "ReadOnlyAttribute"
+    SHARE_ALREADY_EXISTS = "ShareAlreadyExists"
+    SHARE_BEING_DELETED = "ShareBeingDeleted"
+    SHARE_DISABLED = "ShareDisabled"
+    SHARE_NOT_FOUND = "ShareNotFound"
+    SHARING_VIOLATION = "SharingViolation"
+    SHARE_SNAPSHOT_IN_PROGRESS = "ShareSnapshotInProgress"
+    SHARE_SNAPSHOT_COUNT_EXCEEDED = "ShareSnapshotCountExceeded"
+    SHARE_SNAPSHOT_OPERATION_NOT_SUPPORTED = "ShareSnapshotOperationNotSupported"
+    SHARE_HAS_SNAPSHOTS = "ShareHasSnapshots"
+    CONTAINER_QUOTA_DOWNGRADE_NOT_ALLOWED = "ContainerQuotaDowngradeNotAllowed"
 
     # DataLake values
-    content_length_must_be_zero = 'ContentLengthMustBeZero'
-    path_already_exists = 'PathAlreadyExists'
-    invalid_flush_position = 'InvalidFlushPosition'
-    invalid_property_name = 'InvalidPropertyName'
-    invalid_source_uri = 'InvalidSourceUri'
-    unsupported_rest_version = 'UnsupportedRestVersion'
-    file_system_not_found = 'FilesystemNotFound'
-    path_not_found = 'PathNotFound'
-    rename_destination_parent_path_not_found = 'RenameDestinationParentPathNotFound'
-    source_path_not_found = 'SourcePathNotFound'
-    destination_path_is_being_deleted = 'DestinationPathIsBeingDeleted'
-    file_system_already_exists = 'FilesystemAlreadyExists'
-    file_system_being_deleted = 'FilesystemBeingDeleted'
-    invalid_destination_path = 'InvalidDestinationPath'
-    invalid_rename_source_path = 'InvalidRenameSourcePath'
-    invalid_source_or_destination_resource_type = 'InvalidSourceOrDestinationResourceType'
-    lease_is_already_broken = 'LeaseIsAlreadyBroken'
-    lease_name_mismatch = 'LeaseNameMismatch'
-    path_conflict = 'PathConflict'
-    source_path_is_being_deleted = 'SourcePathIsBeingDeleted'
+    CONTENT_LENGTH_MUST_BE_ZERO = 'ContentLengthMustBeZero'
+    PATH_ALREADY_EXISTS = 'PathAlreadyExists'
+    INVALID_FLUSH_POSITION = 'InvalidFlushPosition'
+    INVALID_PROPERTY_NAME = 'InvalidPropertyName'
+    INVALID_SOURCE_URI = 'InvalidSourceUri'
+    UNSUPPORTED_REST_VERSION = 'UnsupportedRestVersion'
+    FILE_SYSTEM_NOT_FOUND = 'FilesystemNotFound'
+    PATH_NOT_FOUND = 'PathNotFound'
+    RENAME_DESTINATION_PARENT_PATH_NOT_FOUND = 'RenameDestinationParentPathNotFound'
+    SOURCE_PATH_NOT_FOUND = 'SourcePathNotFound'
+    DESTINATION_PATH_IS_BEING_DELETED = 'DestinationPathIsBeingDeleted'
+    FILE_SYSTEM_ALREADY_EXISTS = 'FilesystemAlreadyExists'
+    FILE_SYSTEM_BEING_DELETED = 'FilesystemBeingDeleted'
+    INVALID_DESTINATION_PATH = 'InvalidDestinationPath'
+    INVALID_RENAME_SOURCE_PATH = 'InvalidRenameSourcePath'
+    INVALID_SOURCE_OR_DESTINATION_RESOURCE_TYPE = 'InvalidSourceOrDestinationResourceType'
+    LEASE_IS_ALREADY_BROKEN = 'LeaseIsAlreadyBroken'
+    LEASE_NAME_MISMATCH = 'LeaseNameMismatch'
+    PATH_CONFLICT = 'PathConflict'
+    SOURCE_PATH_IS_BEING_DELETED = 'SourcePathIsBeingDeleted'
 
 
 class DictMixin(object):
@@ -328,6 +334,12 @@ class AccountSasPermissions(object):
         To enable set or get tags on the blobs in the container.
     :keyword bool filter_by_tags:
         To enable get blobs by tags, this should be used together with list permission.
+    :keyword bool set_immutability_policy:
+        To enable operations related to set/delete immutability policy.
+        To get immutability policy, you just need read permission.
+    :keyword bool permanent_delete:
+        To enable permanent delete on the blob is permitted.
+        Valid for Object resource type of Blob only.
     """
     def __init__(self, read=False, write=False, delete=False,
                  list=False,  # pylint: disable=redefined-builtin
@@ -336,6 +348,7 @@ class AccountSasPermissions(object):
         self.write = write
         self.delete = delete
         self.delete_previous_version = delete_previous_version
+        self.permanent_delete = kwargs.pop('permanent_delete', False)
         self.list = list
         self.add = add
         self.create = create
@@ -343,17 +356,20 @@ class AccountSasPermissions(object):
         self.process = process
         self.tag = kwargs.pop('tag', False)
         self.filter_by_tags = kwargs.pop('filter_by_tags', False)
+        self.set_immutability_policy = kwargs.pop('set_immutability_policy', False)
         self._str = (('r' if self.read else '') +
                      ('w' if self.write else '') +
                      ('d' if self.delete else '') +
                      ('x' if self.delete_previous_version else '') +
+                     ('y' if self.permanent_delete else '') +
                      ('l' if self.list else '') +
                      ('a' if self.add else '') +
                      ('c' if self.create else '') +
                      ('u' if self.update else '') +
                      ('p' if self.process else '') +
                      ('f' if self.filter_by_tags else '') +
-                     ('t' if self.tag else '')
+                     ('t' if self.tag else '') +
+                     ('i' if self.set_immutability_policy else '')
                      )
 
     def __str__(self):
@@ -376,6 +392,7 @@ class AccountSasPermissions(object):
         p_write = 'w' in permission
         p_delete = 'd' in permission
         p_delete_previous_version = 'x' in permission
+        p_permanent_delete = 'y' in permission
         p_list = 'l' in permission
         p_add = 'a' in permission
         p_create = 'c' in permission
@@ -383,12 +400,15 @@ class AccountSasPermissions(object):
         p_process = 'p' in permission
         p_tag = 't' in permission
         p_filter_by_tags = 'f' in permission
+        p_set_immutability_policy = 'i' in permission
         parsed = cls(read=p_read, write=p_write, delete=p_delete, delete_previous_version=p_delete_previous_version,
                      list=p_list, add=p_add, create=p_create, update=p_update, process=p_process, tag=p_tag,
-                     filter_by_tags=p_filter_by_tags)
+                     filter_by_tags=p_filter_by_tags, set_immutability_policy=p_set_immutability_policy,
+                     permanent_delete=p_permanent_delete)
 
         return parsed
 
+
 class Services(object):
     """Specifies the services accessible with the account SAS.
 
