@@ -3,16 +3,9 @@
 # Licensed under the MIT License. See License.txt in the project root for
 # license information.
 # --------------------------------------------------------------------------
-from typing import (  # pylint: disable=unused-import
-    Union, Optional, Any, TYPE_CHECKING
-)
 
-from azure.multiapi.storagev2.blob.v2021_06_08 import generate_account_sas as generate_blob_account_sas
-from azure.multiapi.storagev2.blob.v2021_06_08 import generate_container_sas, generate_blob_sas
-if TYPE_CHECKING:
-    from datetime import datetime
-    from ._models import AccountSasPermissions, FileSystemSasPermissions, FileSasPermissions, ResourceTypes, \
-        UserDelegationKey
+from azure.storage.blob import generate_account_sas as generate_blob_account_sas
+from azure.storage.blob import generate_container_sas, generate_blob_sas
 
 
 def generate_account_sas(
@@ -21,12 +14,14 @@ def generate_account_sas(
         resource_types,  # type: Union[ResourceTypes, str]
         permission,  # type: Union[AccountSasPermissions, str]
         expiry,  # type: Optional[Union[datetime, str]]
+        start=None,  # type: Optional[Union[datetime, str]]
+        ip=None,  # type: Optional[str]
         **kwargs # type: Any
     ):  # type: (...) -> str
-    """Generates a shared access signature for the DataLake service.
+    """Generates a shared access signature for the blob service.
 
-    Use the returned signature as the credential parameter of any DataLakeServiceClient,
-    FileSystemClient, DataLakeDirectoryClient or DataLakeFileClient.
+    Use the returned signature with the credential parameter of any BlobServiceClient,
+    ContainerClient or BlobClient.
 
     :param str account_name:
         The storage account name used to generate the shared access signature.
@@ -34,14 +29,14 @@ def generate_account_sas(
         The access key to generate the shared access signature.
     :param resource_types:
         Specifies the resource types that are accessible with the account SAS.
-    :type resource_types: str or ~azure.storage.filedatalake.ResourceTypes
+    :type resource_types: str or ~azure.storage.blob.ResourceTypes
     :param permission:
         The permissions associated with the shared access signature. The
         user is restricted to operations allowed by the permissions.
         Required unless an id is given referencing a stored access policy
         which contains this field. This field must be omitted if it has been
         specified in an associated stored access policy.
-    :type permission: str or ~azure.storage.filedatalake.AccountSasPermissions
+    :type permission: str or ~azure.storage.blob.AccountSasPermissions
     :param expiry:
         The time at which the shared access signature becomes invalid.
         Required unless an id is given referencing a stored access policy
@@ -50,14 +45,14 @@ def generate_account_sas(
         convert values to UTC. If a date is passed in without timezone info, it
         is assumed to be UTC.
     :type expiry: ~datetime.datetime or str
-    :keyword start:
+    :param start:
         The time at which the shared access signature becomes valid. If
         omitted, start time for this call is assumed to be the time when the
         storage service receives the request. Azure will always convert values
         to UTC. If a date is passed in without timezone info, it is assumed to
         be UTC.
-    :paramtype start: ~datetime.datetime or str
-    :keyword str ip:
+    :type start: ~datetime.datetime or str
+    :param str ip:
         Specifies an IP address or a range of IP addresses from which to accept requests.
         If the IP address from which the request originates does not match the IP address
         or address range specified on the SAS token, the request is not authenticated.
@@ -67,6 +62,15 @@ def generate_account_sas(
         Specifies the protocol permitted for a request made. The default value is https.
     :return: A Shared Access Signature (sas) token.
     :rtype: str
+
+    .. admonition:: Example:
+
+        .. literalinclude:: ../tests/test_blob_samples_authentication.py
+            :start-after: [START create_sas_token]
+            :end-before: [END create_sas_token]
+            :language: python
+            :dedent: 8
+            :caption: Generating a shared access signature.
     """
     return generate_blob_account_sas(
         account_name=account_name,
@@ -74,6 +78,8 @@ def generate_account_sas(
         resource_types=resource_types,
         permission=permission,
         expiry=expiry,
+        start=start,
+        ip=ip,
         **kwargs
     )
 
@@ -81,38 +87,40 @@ def generate_account_sas(
 def generate_file_system_sas(
         account_name,  # type: str
         file_system_name,  # type: str
-        credential,  # type: Union[str, UserDelegationKey]
+        account_key=None,  # type: Optional[str]
+        user_delegation_key=None,  # type: Optional[UserDelegationKey]
         permission=None,  # type: Optional[Union[FileSystemSasPermissions, str]]
         expiry=None,  # type: Optional[Union[datetime, str]]
+        start=None,  # type: Optional[Union[datetime, str]]
+        ip=None,  # type: Optional[str]
         **kwargs # type: Any
     ):
     # type: (...) -> str
-    """Generates a shared access signature for a file system.
+    """Generates a shared access signature for a container.
 
-    Use the returned signature with the credential parameter of any DataLakeServiceClient,
-    FileSystemClient, DataLakeDirectoryClient or DataLakeFileClient.
+    Use the returned signature with the credential parameter of any BlobServiceClient,
+    ContainerClient or BlobClient.
 
     :param str account_name:
         The storage account name used to generate the shared access signature.
     :param str file_system_name:
         The name of the file system.
-    :param str credential:
-        Credential could be either account key or user delegation key.
-        If use account key is used as credential, then the credential type should be a str.
-        Instead of an account key, the user could also pass in a user delegation key.
+    :param str account_key:
+        The access key to generate the shared access signature. Either `account_key` or
+        `user_delegation_key` must be specified.
+    :param ~azure.storage.blob.UserDelegationKey user_delegation_key:
+        Instead of an account key, the user could pass in a user delegation key.
         A user delegation key can be obtained from the service by authenticating with an AAD identity;
-        this can be accomplished
-        by calling :func:`~azure.storage.filedatalake.DataLakeServiceClient.get_user_delegation_key`.
+        this can be accomplished by calling :func:`~azure.storage.blob.BlobServiceClient.get_user_delegation_key`.
         When present, the SAS is signed with the user delegation key instead.
-    :type credential: str or ~azure.storage.filedatalake.UserDelegationKey
     :param permission:
         The permissions associated with the shared access signature. The
         user is restricted to operations allowed by the permissions.
-        Permissions must be ordered racwdlmeop.
+        Permissions must be ordered read, write, delete, list.
         Required unless an id is given referencing a stored access policy
         which contains this field. This field must be omitted if it has been
         specified in an associated stored access policy.
-    :type permission: str or ~azure.storage.filedatalake.FileSystemSasPermissions
+    :type permission: str or ~azure.storage.blob.ContainerSasPermissions
     :param expiry:
         The time at which the shared access signature becomes invalid.
         Required unless an id is given referencing a stored access policy
@@ -121,14 +129,14 @@ def generate_file_system_sas(
         convert values to UTC. If a date is passed in without timezone info, it
         is assumed to be UTC.
     :type expiry: datetime or str
-    :keyword start:
+    :param start:
         The time at which the shared access signature becomes valid. If
         omitted, start time for this call is assumed to be the time when the
         storage service receives the request. Azure will always convert values
         to UTC. If a date is passed in without timezone info, it is assumed to
         be UTC.
-    :paramtype start: datetime or str
-    :keyword str ip:
+    :type start: datetime or str
+    :param str ip:
         Specifies an IP address or a range of IP addresses from which to accept requests.
         If the IP address from which the request originates does not match the IP address
         or address range specified on the SAS token, the request is not authenticated.
@@ -151,29 +159,27 @@ def generate_file_system_sas(
     :keyword str content_type:
         Response header value for Content-Type when resource is accessed
         using this shared access signature.
-    :keyword str preauthorized_agent_object_id:
-        The AAD object ID of a user assumed to be authorized by the owner of the user delegation key to perform
-        the action granted by the SAS token. The service will validate the SAS token and ensure that the owner of the
-        user delegation key has the required permissions before granting access but no additional permission check for
-        the agent object id will be performed.
-    :keyword str agent_object_id:
-        The AAD object ID of a user assumed to be unauthorized by the owner of the user delegation key to
-        perform the action granted by the SAS token. The service will validate the SAS token and ensure that the owner
-        of the user delegation key has the required permissions before granting access and the service will perform an
-        additional POSIX ACL check to determine if this user is authorized to perform the requested operation.
-    :keyword str correlation_id:
-        The correlation id to correlate the storage audit logs with the audit logs used by the principal
-        generating and distributing the SAS.
     :return: A Shared Access Signature (sas) token.
     :rtype: str
+
+    .. admonition:: Example:
+
+        .. literalinclude:: ../tests/test_blob_samples_containers.py
+            :start-after: [START generate_sas_token]
+            :end-before: [END generate_sas_token]
+            :language: python
+            :dedent: 12
+            :caption: Generating a sas token.
     """
     return generate_container_sas(
         account_name=account_name,
         container_name=file_system_name,
-        account_key=credential if isinstance(credential, str) else None,
-        user_delegation_key=credential if not isinstance(credential, str) else None,
+        account_key=account_key,
+        user_delegation_key=user_delegation_key,
         permission=permission,
         expiry=expiry,
+        start=start,
+        ip=ip,
         **kwargs)
 
 
@@ -181,40 +187,44 @@ def generate_directory_sas(
         account_name,  # type: str
         file_system_name,  # type: str
         directory_name,  # type: str
-        credential,  # type: Union[str, UserDelegationKey]
-        permission=None,  # type: Optional[Union[FileSasPermissions, str]]
+        account_key=None,  # type: Optional[str]
+        user_delegation_key=None,  # type: Optional[UserDelegationKey]
+        permission=None,  # type: Optional[Union[BlobSasPermissions, str]]
         expiry=None,  # type: Optional[Union[datetime, str]]
+        start=None,  # type: Optional[Union[datetime, str]]
+        ip=None,  # type: Optional[str]
         **kwargs # type: Any
     ):
     # type: (...) -> str
-    """Generates a shared access signature for a directory.
+    """Generates a shared access signature for a blob.
 
-    Use the returned signature with the credential parameter of any DataLakeServiceClient,
-    FileSystemClient, DataLakeDirectoryClient or DataLakeFileClient.
+    Use the returned signature with the credential parameter of any BlobServiceClient,
+    ContainerClient or BlobClient.
 
     :param str account_name:
         The storage account name used to generate the shared access signature.
-    :param str file_system_name:
-        The name of the file system.
-    :param str directory_name:
-        The name of the directory.
-    :param str credential:
-        Credential could be either account key or user delegation key.
-        If use account key is used as credential, then the credential type should be a str.
-        Instead of an account key, the user could also pass in a user delegation key.
+    :param str container_name:
+        The name of the container.
+    :param str blob_name:
+        The name of the blob.
+    :param str snapshot:
+        An optional blob snapshot ID.
+    :param str account_key:
+        The access key to generate the shared access signature. Either `account_key` or
+        `user_delegation_key` must be specified.
+    :param ~azure.storage.blob.UserDelegationKey user_delegation_key:
+        Instead of an account key, the user could pass in a user delegation key.
         A user delegation key can be obtained from the service by authenticating with an AAD identity;
-        this can be accomplished
-        by calling :func:`~azure.storage.filedatalake.DataLakeServiceClient.get_user_delegation_key`.
+        this can be accomplished by calling :func:`~azure.storage.blob.BlobServiceClient.get_user_delegation_key`.
         When present, the SAS is signed with the user delegation key instead.
-    :type credential: str or ~azure.storage.filedatalake.UserDelegationKey
     :param permission:
         The permissions associated with the shared access signature. The
         user is restricted to operations allowed by the permissions.
-        Permissions must be ordered racwdlmeop.
+        Permissions must be ordered read, write, delete, list.
         Required unless an id is given referencing a stored access policy
         which contains this field. This field must be omitted if it has been
         specified in an associated stored access policy.
-    :type permission: str or ~azure.storage.filedatalake.FileSasPermissions
+    :type permission: str or ~azure.storage.blob.BlobSasPermissions
     :param expiry:
         The time at which the shared access signature becomes invalid.
         Required unless an id is given referencing a stored access policy
@@ -223,14 +233,14 @@ def generate_directory_sas(
         convert values to UTC. If a date is passed in without timezone info, it
         is assumed to be UTC.
     :type expiry: ~datetime.datetime or str
-    :keyword start:
+    :param start:
         The time at which the shared access signature becomes valid. If
         omitted, start time for this call is assumed to be the time when the
         storage service receives the request. Azure will always convert values
         to UTC. If a date is passed in without timezone info, it is assumed to
         be UTC.
-    :paramtype start: ~datetime.datetime or str
-    :keyword str ip:
+    :type start: ~datetime.datetime or str
+    :param str ip:
         Specifies an IP address or a range of IP addresses from which to accept requests.
         If the IP address from which the request originates does not match the IP address
         or address range specified on the SAS token, the request is not authenticated.
@@ -253,33 +263,19 @@ def generate_directory_sas(
     :keyword str content_type:
         Response header value for Content-Type when resource is accessed
         using this shared access signature.
-    :keyword str preauthorized_agent_object_id:
-        The AAD object ID of a user assumed to be authorized by the owner of the user delegation key to perform
-        the action granted by the SAS token. The service will validate the SAS token and ensure that the owner of the
-        user delegation key has the required permissions before granting access but no additional permission check for
-        the agent object id will be performed.
-    :keyword str agent_object_id:
-        The AAD object ID of a user assumed to be unauthorized by the owner of the user delegation key to
-        perform the action granted by the SAS token. The service will validate the SAS token and ensure that the owner
-        of the user delegation key has the required permissions before granting access and the service will perform an
-        additional POSIX ACL check to determine if this user is authorized to perform the requested operation.
-    :keyword str correlation_id:
-        The correlation id to correlate the storage audit logs with the audit logs used by the principal
-        generating and distributing the SAS.
     :return: A Shared Access Signature (sas) token.
     :rtype: str
     """
-    depth = len(directory_name.strip("/").split("/"))
     return generate_blob_sas(
         account_name=account_name,
         container_name=file_system_name,
         blob_name=directory_name,
-        account_key=credential if isinstance(credential, str) else None,
-        user_delegation_key=credential if not isinstance(credential, str) else None,
+        account_key=account_key,
+        user_delegation_key=user_delegation_key,
         permission=permission,
         expiry=expiry,
-        sdd=depth,
-        is_directory=True,
+        start=start,
+        ip=ip,
         **kwargs)
 
 
@@ -288,42 +284,44 @@ def generate_file_sas(
         file_system_name,  # type: str
         directory_name,  # type: str
         file_name,  # type: str
-        credential,  # type: Union[str, UserDelegationKey]
-        permission=None,  # type: Optional[Union[FileSasPermissions, str]]
+        account_key=None,  # type: Optional[str]
+        user_delegation_key=None,  # type: Optional[UserDelegationKey]
+        permission=None,  # type: Optional[Union[BlobSasPermissions, str]]
         expiry=None,  # type: Optional[Union[datetime, str]]
-        **kwargs  # type: Any
+        start=None,  # type: Optional[Union[datetime, str]]
+        ip=None,  # type: Optional[str]
+        **kwargs # type: Any
     ):
     # type: (...) -> str
-    """Generates a shared access signature for a file.
+    """Generates a shared access signature for a blob.
 
-    Use the returned signature with the credential parameter of any BDataLakeServiceClient,
-    FileSystemClient, DataLakeDirectoryClient or DataLakeFileClient.
+    Use the returned signature with the credential parameter of any BlobServiceClient,
+    ContainerClient or BlobClient.
 
     :param str account_name:
         The storage account name used to generate the shared access signature.
-    :param str file_system_name:
-        The name of the file system.
-    :param str directory_name:
-        The name of the directory.
-    :param str file_name:
-        The name of the file.
-    :param str credential:
-        Credential could be either account key or user delegation key.
-        If use account key is used as credential, then the credential type should be a str.
-        Instead of an account key, the user could also pass in a user delegation key.
+    :param str container_name:
+        The name of the container.
+    :param str blob_name:
+        The name of the blob.
+    :param str snapshot:
+        An optional blob snapshot ID.
+    :param str account_key:
+        The access key to generate the shared access signature. Either `account_key` or
+        `user_delegation_key` must be specified.
+    :param ~azure.storage.blob.UserDelegationKey user_delegation_key:
+        Instead of an account key, the user could pass in a user delegation key.
         A user delegation key can be obtained from the service by authenticating with an AAD identity;
-        this can be accomplished
-        by calling :func:`~azure.storage.filedatalake.DataLakeServiceClient.get_user_delegation_key`.
+        this can be accomplished by calling :func:`~azure.storage.blob.BlobServiceClient.get_user_delegation_key`.
         When present, the SAS is signed with the user delegation key instead.
-    :type credential: str or ~azure.storage.filedatalake.UserDelegationKey
     :param permission:
         The permissions associated with the shared access signature. The
         user is restricted to operations allowed by the permissions.
-        Permissions must be ordered racwdmeop.
+        Permissions must be ordered read, write, delete, list.
         Required unless an id is given referencing a stored access policy
         which contains this field. This field must be omitted if it has been
         specified in an associated stored access policy.
-    :type permission: str or ~azure.storage.filedatalake.FileSasPermissions
+    :type permission: str or ~azure.storage.blob.BlobSasPermissions
     :param expiry:
         The time at which the shared access signature becomes invalid.
         Required unless an id is given referencing a stored access policy
@@ -332,14 +330,14 @@ def generate_file_sas(
         convert values to UTC. If a date is passed in without timezone info, it
         is assumed to be UTC.
     :type expiry: ~datetime.datetime or str
-    :keyword start:
+    :param start:
         The time at which the shared access signature becomes valid. If
         omitted, start time for this call is assumed to be the time when the
         storage service receives the request. Azure will always convert values
         to UTC. If a date is passed in without timezone info, it is assumed to
         be UTC.
-    :paramtype start: ~datetime.datetime or str
-    :keyword str ip:
+    :type start: ~datetime.datetime or str
+    :param str ip:
         Specifies an IP address or a range of IP addresses from which to accept requests.
         If the IP address from which the request originates does not match the IP address
         or address range specified on the SAS token, the request is not authenticated.
@@ -362,19 +360,6 @@ def generate_file_sas(
     :keyword str content_type:
         Response header value for Content-Type when resource is accessed
         using this shared access signature.
-    :keyword str preauthorized_agent_object_id:
-        The AAD object ID of a user assumed to be authorized by the owner of the user delegation key to perform
-        the action granted by the SAS token. The service will validate the SAS token and ensure that the owner of the
-        user delegation key has the required permissions before granting access but no additional permission check for
-        the agent object id will be performed.
-    :keyword str agent_object_id:
-        The AAD object ID of a user assumed to be unauthorized by the owner of the user delegation key to
-        perform the action granted by the SAS token. The service will validate the SAS token and ensure that the owner
-        of the user delegation key has the required permissions before granting access and the service will perform an
-        additional POSIX ACL check to determine if this user is authorized to perform the requested operation.
-    :keyword str correlation_id:
-        The correlation id to correlate the storage audit logs with the audit logs used by the principal
-        generating and distributing the SAS. This can only be used when to generate sas with delegation key.
     :return: A Shared Access Signature (sas) token.
     :rtype: str
     """
@@ -386,8 +371,10 @@ def generate_file_sas(
         account_name=account_name,
         container_name=file_system_name,
         blob_name=path,
-        account_key=credential if isinstance(credential, str) else None,
-        user_delegation_key=credential if not isinstance(credential, str) else None,
+        account_key=account_key,
+        user_delegation_key=user_delegation_key,
         permission=permission,
         expiry=expiry,
+        start=start,
+        ip=ip,
         **kwargs)
