@@ -6,21 +6,14 @@
 # pylint: disable=too-few-public-methods, too-many-instance-attributes
 # pylint: disable=super-init-not-called, too-many-lines
 
-from enum import Enum
-
-from azure.core import CaseInsensitiveEnumMeta
 from azure.core.paging import PageIterator
-from azure.core.exceptions import HttpResponseError
-
 from ._parser import _parse_datetime_from_str
 from ._shared.response_handlers import return_context_and_deserialized, process_storage_error
 from ._shared.models import DictMixin, get_enum_value
+from ._generated.models import StorageErrorException
 from ._generated.models import Metrics as GeneratedMetrics
 from ._generated.models import RetentionPolicy as GeneratedRetentionPolicy
 from ._generated.models import CorsRule as GeneratedCorsRule
-from ._generated.models import ShareProtocolSettings as GeneratedShareProtocolSettings
-from ._generated.models import ShareSmbSettings as GeneratedShareSmbSettings
-from ._generated.models import SmbMultichannel as GeneratedSmbMultichannel
 from ._generated.models import AccessPolicy as GenAccessPolicy
 from ._generated.models import DirectoryItem
 
@@ -141,46 +134,6 @@ class CorsRule(GeneratedCorsRule):
         )
 
 
-class ShareSmbSettings(GeneratedShareSmbSettings):
-    """ Settings for the SMB protocol.
-
-    :keyword SmbMultichannel multichannel: Sets the multichannel settings.
-    """
-    def __init__(self, **kwargs):
-        self.multichannel = kwargs.get('multichannel')
-        if self.multichannel is None:
-            raise ValueError("The value 'multichannel' must be specified.")
-
-
-class SmbMultichannel(GeneratedSmbMultichannel):
-    """ Settings for Multichannel.
-
-    :keyword bool enabled: If SMB Multichannel is enabled.
-    """
-    def __init__(self, **kwargs):
-        self.enabled = kwargs.get('enabled')
-        if self.enabled is None:
-            raise ValueError("The value 'enabled' must be specified.")
-
-
-class ShareProtocolSettings(GeneratedShareProtocolSettings):
-    """Protocol Settings class used by the set and get service properties methods in the share service.
-
-    Contains protocol properties of the share service such as the SMB setting of the share service.
-
-    :keyword SmbSettings smb: Sets SMB settings.
-    """
-    def __init__(self, **kwargs):
-        self.smb = kwargs.get('smb')
-        if self.smb is None:
-            raise ValueError("The value 'smb' must be specified.")
-
-    @classmethod
-    def _from_generated(cls, generated):
-        return cls(
-            smb=generated.smb)
-
-
 class AccessPolicy(GenAccessPolicy):
     """Access Policy class used by the set and get acl methods in each service.
 
@@ -232,14 +185,14 @@ class AccessPolicy(GenAccessPolicy):
 
 
 class LeaseProperties(DictMixin):
-    """File or Share Lease Properties.
+    """File Lease Properties.
 
     :ivar str status:
-        The lease status of the file or share. Possible values: locked|unlocked
+        The lease status of the file. Possible values: locked|unlocked
     :ivar str state:
-        Lease state of the file or share. Possible values: available|leased|expired|breaking|broken
+        Lease state of the file. Possible values: available|leased|expired|breaking|broken
     :ivar str duration:
-        When a file or share is leased, specifies whether the lease is of infinite or fixed duration.
+        When a file is leased, specifies whether the lease is of infinite or fixed duration.
     """
 
     def __init__(self, **kwargs):
@@ -276,7 +229,7 @@ class ContentSettings(DictMixin):
     :param str cache_control:
         If the cache_control has previously been set for
         the file, that value is stored.
-    :param bytearray content_md5:
+    :param str content_md5:
         If the content_md5 has been set for the file, this response
         header is stored so that the client can check for message content
         integrity.
@@ -318,30 +271,10 @@ class ShareProperties(DictMixin):
         conditionally.
     :ivar int quota:
         The allocated quota.
-    :ivar str access_tier:
-        The share's access tier.
     :ivar dict metadata: A dict with name_value pairs to associate with the
         share as metadata.
     :ivar str snapshot:
         Snapshot of the share.
-    :ivar bool deleted:
-        To indicate if this share is deleted or not.
-        This is a service returned value, and the value will be set when list shared including deleted ones.
-    :ivar datetime deleted:
-        To indicate the deleted time of the deleted share.
-        This is a service returned value, and the value will be set when list shared including deleted ones.
-    :ivar str version:
-        To indicate the version of deleted share.
-        This is a service returned value, and the value will be set when list shared including deleted ones.
-    :ivar int remaining_retention_days:
-        To indicate how many remaining days the deleted share will be kept.
-        This is a service returned value, and the value will be set when list shared including deleted ones.
-    :ivar int provisioned_bandwidth:
-        Provisioned bandwidth in megabits/second. Only applicable to premium file accounts.
-    :ivar ~azure.storage.fileshare.models.ShareRootSquash or str root_squash:
-        Possible values include: 'NoRootSquash', 'RootSquash', 'AllSquash'.
-    :ivar list(str) protocols:
-        Indicates the protocols enabled on the share. The protocol can be either SMB or NFS.
     """
 
     def __init__(self, **kwargs):
@@ -349,22 +282,13 @@ class ShareProperties(DictMixin):
         self.last_modified = kwargs.get('Last-Modified')
         self.etag = kwargs.get('ETag')
         self.quota = kwargs.get('x-ms-share-quota')
-        self.access_tier = kwargs.get('x-ms-access-tier')
         self.next_allowed_quota_downgrade_time = kwargs.get('x-ms-share-next-allowed-quota-downgrade-time')
         self.metadata = kwargs.get('metadata')
         self.snapshot = None
-        self.deleted = None
-        self.deleted_time = None
-        self.version = None
-        self.remaining_retention_days = None
         self.provisioned_egress_mbps = kwargs.get('x-ms-share-provisioned-egress-mbps')
         self.provisioned_ingress_mbps = kwargs.get('x-ms-share-provisioned-ingress-mbps')
         self.provisioned_iops = kwargs.get('x-ms-share-provisioned-iops')
-        self.provisioned_bandwidth = kwargs.get('x-ms-share-provisioned-bandwidth-mibps')
-        self.lease = LeaseProperties(**kwargs)
-        self.protocols = [protocol.strip() for protocol in kwargs.get('x-ms-enabled-protocols', None).split(',')]\
-            if kwargs.get('x-ms-enabled-protocols', None) else None
-        self.root_squash = kwargs.get('x-ms-root-squash', None)
+
     @classmethod
     def _from_generated(cls, generated):
         props = cls()
@@ -372,23 +296,12 @@ class ShareProperties(DictMixin):
         props.last_modified = generated.properties.last_modified
         props.etag = generated.properties.etag
         props.quota = generated.properties.quota
-        props.access_tier = generated.properties.access_tier
         props.next_allowed_quota_downgrade_time = generated.properties.next_allowed_quota_downgrade_time
         props.metadata = generated.metadata
         props.snapshot = generated.snapshot
-        props.deleted = generated.deleted
-        props.deleted_time = generated.properties.deleted_time
-        props.version = generated.version
-        props.remaining_retention_days = generated.properties.remaining_retention_days
-        props.provisioned_egress_mbps = generated.properties.provisioned_egress_m_bps
-        props.provisioned_ingress_mbps = generated.properties.provisioned_ingress_m_bps
+        props.provisioned_egress_mbps = generated.properties.provisioned_egress_mbps
+        props.provisioned_ingress_mbps = generated.properties.provisioned_ingress_mbps
         props.provisioned_iops = generated.properties.provisioned_iops
-        props.provisioned_bandwidth = generated.properties.provisioned_bandwidth_mi_bps
-        props.lease = LeaseProperties._from_generated(generated)  # pylint: disable=protected-access
-        props.protocols = [protocol.strip() for protocol in generated.properties.enabled_protocols.split(',')]\
-            if generated.properties.enabled_protocols else None
-        props.root_squash = generated.properties.root_squash
-
         return props
 
 
@@ -434,7 +347,7 @@ class SharePropertiesPaged(PageIterator):
                 prefix=self.prefix,
                 cls=return_context_and_deserialized,
                 use_location=self.location_mode)
-        except HttpResponseError as error:
+        except StorageErrorException as error:
             process_storage_error(error)
 
     def _extract_data_cb(self, get_next_return):
@@ -526,7 +439,7 @@ class HandlesPaged(PageIterator):
                 maxresults=self.results_per_page,
                 cls=return_context_and_deserialized,
                 use_location=self.location_mode)
-        except HttpResponseError as error:
+        except StorageErrorException as error:
             process_storage_error(error)
 
     def _extract_data_cb(self, get_next_return):
@@ -555,8 +468,6 @@ class DirectoryProperties(DictMixin):
     :vartype creation_time: str or ~datetime.datetime
     :ivar last_write_time: Last write time for the file.
     :vartype last_write_time: str or ~datetime.datetime
-    :ivar last_access_time: Last access time for the file.
-    :vartype last_access_time: ~datetime.datetime
     :ivar file_attributes:
         The file system attributes for files and directories.
     :vartype file_attributes: str or :class:`~azure.storage.fileshare.NTFSAttributes`
@@ -580,26 +491,19 @@ class DirectoryProperties(DictMixin):
         self.change_time = _parse_datetime_from_str(kwargs.get('x-ms-file-change-time'))
         self.creation_time = _parse_datetime_from_str(kwargs.get('x-ms-file-creation-time'))
         self.last_write_time = _parse_datetime_from_str(kwargs.get('x-ms-file-last-write-time'))
-        self.last_access_time = None
         self.file_attributes = kwargs.get('x-ms-file-attributes')
         self.permission_key = kwargs.get('x-ms-file-permission-key')
         self.file_id = kwargs.get('x-ms-file-id')
         self.parent_id = kwargs.get('x-ms-file-parent-id')
-        self.is_directory = True
 
     @classmethod
     def _from_generated(cls, generated):
         props = cls()
         props.name = generated.name
-        props.file_id = generated.file_id
-        props.file_attributes = generated.attributes
         props.last_modified = generated.properties.last_modified
-        props.creation_time = generated.properties.creation_time
-        props.last_access_time = generated.properties.last_access_time
-        props.last_write_time = generated.properties.last_write_time
-        props.change_time = generated.properties.change_time
         props.etag = generated.properties.etag
-        props.permission_key = generated.permission_key
+        props.server_encrypted = generated.properties.server_encrypted
+        props.metadata = generated.metadata
         return props
 
 
@@ -649,7 +553,7 @@ class DirectoryPropertiesPaged(PageIterator):
                 maxresults=self.results_per_page,
                 cls=return_context_and_deserialized,
                 use_location=self.location_mode)
-        except HttpResponseError as error:
+        except StorageErrorException as error:
             process_storage_error(error)
 
     def _extract_data_cb(self, get_next_return):
@@ -658,8 +562,8 @@ class DirectoryPropertiesPaged(PageIterator):
         self.prefix = self._response.prefix
         self.marker = self._response.marker
         self.results_per_page = self._response.max_results
-        self.current_page = [DirectoryProperties._from_generated(i) for i in self._response.segment.directory_items] # pylint: disable = protected-access
-        self.current_page.extend([FileProperties._from_generated(i) for i in self._response.segment.file_items]) # pylint: disable = protected-access
+        self.current_page = [_wrap_item(i) for i in self._response.segment.directory_items]
+        self.current_page.extend([_wrap_item(i) for i in self._response.segment.file_items])
         return self._response.next_marker or None, self.current_page
 
 
@@ -718,36 +622,21 @@ class FileProperties(DictMixin):
         self.change_time = _parse_datetime_from_str(kwargs.get('x-ms-file-change-time'))
         self.creation_time = _parse_datetime_from_str(kwargs.get('x-ms-file-creation-time'))
         self.last_write_time = _parse_datetime_from_str(kwargs.get('x-ms-file-last-write-time'))
-        self.last_access_time = None
         self.file_attributes = kwargs.get('x-ms-file-attributes')
         self.permission_key = kwargs.get('x-ms-file-permission-key')
         self.file_id = kwargs.get('x-ms-file-id')
         self.parent_id = kwargs.get('x-ms-file-parent-id')
-        self.is_directory = False
 
     @classmethod
     def _from_generated(cls, generated):
         props = cls()
         props.name = generated.name
-        props.file_id = generated.file_id
-        props.etag = generated.properties.etag
-        props.file_attributes = generated.attributes
-        props.last_modified = generated.properties.last_modified
-        props.creation_time = generated.properties.creation_time
-        props.last_access_time = generated.properties.last_access_time
-        props.last_write_time = generated.properties.last_write_time
-        props.change_time = generated.properties.change_time
-        props.size = generated.properties.content_length
-        props.permission_key = generated.permission_key
+        props.content_length = generated.properties.content_length
+        props.metadata = generated.properties.metadata
+        props.lease = LeaseProperties._from_generated(generated)  # pylint: disable=protected-access
         return props
 
 
-class ShareProtocols(str, Enum, metaclass=CaseInsensitiveEnumMeta):
-    """Enabled protocols on the share"""
-    SMB = "SMB"
-    NFS = "NFS"
-
-
 class CopyProperties(DictMixin):
     """File Copy Properties.
 
@@ -864,7 +753,7 @@ class FileSasPermissions(object):
         p_delete = 'd' in permission
 
         parsed = cls(p_read, p_create, p_write, p_delete)
-
+        parsed._str = permission # pylint: disable = protected-access
         return parsed
 
 
@@ -887,17 +776,13 @@ class ShareSasPermissions(object):
         an account SAS instead.
     :param bool list:
         List files and directories in the share.
-    :param bool create:
-        Create a new file in the share, or copy a file to a new file in the share.
     """
-    def __init__(self, read=False, write=False, delete=False, list=False, create=False):  # pylint: disable=redefined-builtin
+    def __init__(self, read=False, write=False, delete=False, list=False):  # pylint: disable=redefined-builtin
         self.read = read
-        self.create = create
         self.write = write
         self.delete = delete
         self.list = list
         self._str = (('r' if self.read else '') +
-                     ('c' if self.create else '') +
                      ('w' if self.write else '') +
                      ('d' if self.delete else '') +
                      ('l' if self.list else ''))
@@ -910,23 +795,22 @@ class ShareSasPermissions(object):
     def from_string(cls, permission):
         """Create a ShareSasPermissions from a string.
 
-        To specify read, create, write, delete, or list permissions you need only to
+        To specify read, write, delete, or list permissions you need only to
         include the first letter of the word in the string. E.g. For read and
         write permissions, you would provide a string "rw".
 
-        :param str permission: The string which dictates the read, create, write,
+        :param str permission: The string which dictates the read, write,
             delete, or list permissions
         :return: A ShareSasPermissions object
         :rtype: ~azure.storage.fileshare.ShareSasPermissions
         """
         p_read = 'r' in permission
-        p_create = 'c' in permission
         p_write = 'w' in permission
         p_delete = 'd' in permission
         p_list = 'l' in permission
 
-        parsed = cls(p_read, p_write, p_delete, p_list, p_create)
-
+        parsed = cls(p_read, p_write, p_delete, p_list)
+        parsed._str = permission # pylint: disable = protected-access
         return parsed
 
 class NTFSAttributes(object):
@@ -1018,5 +902,4 @@ def service_properties_deserialize(generated):
         'hour_metrics': Metrics._from_generated(generated.hour_metrics),  # pylint: disable=protected-access
         'minute_metrics': Metrics._from_generated(generated.minute_metrics),  # pylint: disable=protected-access
         'cors': [CorsRule._from_generated(cors) for cors in generated.cors],  # pylint: disable=protected-access
-        'protocol': ShareProtocolSettings._from_generated(generated.protocol), # pylint: disable=protected-access
     }
