@@ -201,7 +201,7 @@ class StorageStreamDownloader(object):  # pylint: disable=too-many-instance-attr
         The properties of the file being downloaded. If only a range of the data is being
         downloaded, this will be reflected in the properties.
     :ivar int size:
-        The size of the total data in the stream. This will be the byte range if speficied,
+        The size of the total data in the stream. This will be the byte range if specified,
         otherwise the total size of the file.
     """
 
@@ -311,7 +311,7 @@ class StorageStreamDownloader(object):  # pylint: disable=too-many-instance-attr
             # Parse the total file size and adjust the download size if ranges
             # were specified
             self._file_size = parse_length_from_content_range(response.properties.content_range)
-            if not self._file_size:
+            if self._file_size is None:
                 raise ValueError("Required Content-Range response header is missing or malformed.")
 
             if self._end_range is not None:
@@ -323,7 +323,7 @@ class StorageStreamDownloader(object):  # pylint: disable=too-many-instance-attr
                 self.size = self._file_size
 
         except HttpResponseError as error:
-            if self._start_range is None and error.response.status_code == 416:
+            if self._start_range is None and error.response and error.response.status_code == 416:
                 # Get range will fail on an empty file. If the user did not
                 # request a range, do a regular get request in order to get
                 # any properties.
@@ -398,10 +398,12 @@ class StorageStreamDownloader(object):  # pylint: disable=too-many-instance-attr
         return data
 
     def content_as_bytes(self, max_concurrency=1):
-        """Download the contents of this file.
+        """DEPRECATED: Download the contents of this file.
 
         This operation is blocking until all data is downloaded.
 
+        This method is deprecated, use func:`readall` instead.
+
         :keyword int max_concurrency:
             The number of parallel connections with which to download.
         :rtype: bytes
@@ -414,10 +416,12 @@ class StorageStreamDownloader(object):  # pylint: disable=too-many-instance-attr
         return self.readall()
 
     def content_as_text(self, max_concurrency=1, encoding="UTF-8"):
-        """Download the contents of this file, and decode as text.
+        """DEPRECATED: Download the contents of this file, and decode as text.
 
         This operation is blocking until all data is downloaded.
 
+        This method is deprecated, use func:`readall` instead.
+
         :keyword int max_concurrency:
             The number of parallel connections with which to download.
         :param str encoding:
@@ -495,7 +499,9 @@ class StorageStreamDownloader(object):  # pylint: disable=too-many-instance-attr
         return self.size
 
     def download_to_stream(self, stream, max_concurrency=1):
-        """Download the contents of this file to a stream.
+        """DEPRECATED: Download the contents of this file to a stream.
+
+        This method is deprecated, use func:`readinto` instead.
 
         :param stream:
             The stream to download to. This can be an open file-handle,
