@@ -3,41 +3,58 @@
 # Licensed under the MIT License. See License.txt in the project root for
 # license information.
 # --------------------------------------------------------------------------
-# pylint: disable=invalid-overridden-method
 
 import functools
-import warnings
 from typing import (  # pylint: disable=unused-import
-    Any, Dict, List, Optional,
-    TYPE_CHECKING)
+    Union,
+    Optional,
+    Any,
+    IO,
+    Iterable,
+    AnyStr,
+    Dict,
+    List,
+    Tuple,
+    TYPE_CHECKING,
+)
+
+try:
+    from urllib.parse import urlparse, quote, unquote  # pylint: disable=unused-import
+except ImportError:
+    from urlparse import urlparse  # type: ignore
+    from urllib2 import quote, unquote  # type: ignore
 
-from azure.core.async_paging import AsyncItemPaged
-from azure.core.exceptions import HttpResponseError
 from azure.core.tracing.decorator import distributed_trace
 from azure.core.tracing.decorator_async import distributed_trace_async
 
-from .._serialize import get_api_version
+from azure.core.async_paging import AsyncItemPaged
+
 from .._shared.base_client_async import AsyncStorageAccountHostsMixin
-from .._shared.policies_async import ExponentialRetry
 from .._shared.request_handlers import add_metadata_headers, serialize_iso
 from .._shared.response_handlers import (
     return_response_headers,
     process_storage_error,
     return_headers_and_deserialized,
 )
-from .._generated.aio import AzureQueueStorage
-from .._generated.models import SignedIdentifier, QueueMessage as GenQueueMessage
 from .._deserialize import deserialize_queue_properties, deserialize_queue_creation
-from .._encryption import StorageEncryptionMixin
+from .._generated.version import VERSION
+from .._generated.aio import AzureQueueStorage
+from .._generated.models import StorageErrorException, SignedIdentifier
+from .._generated.models import QueueMessage as GenQueueMessage
+
 from .._models import QueueMessage, AccessPolicy
-from .._queue_client import QueueClient as QueueClientBase
 from ._models import MessagesPaged
+from .._shared.policies_async import ExponentialRetry
+from .._queue_client import QueueClient as QueueClientBase
+
 
 if TYPE_CHECKING:
-    from .._models import QueueProperties
+    from datetime import datetime
+    from azure.core.pipeline.policies import HTTPPolicy
+    from .._models import QueueSasPermissions, QueueProperties
 
 
-class QueueClient(AsyncStorageAccountHostsMixin, QueueClientBase, StorageEncryptionMixin):
+class QueueClient(AsyncStorageAccountHostsMixin, QueueClientBase):
     """A client to interact with a specific Queue.
 
     :param str account_url:
@@ -47,22 +64,17 @@ class QueueClient(AsyncStorageAccountHostsMixin, QueueClientBase, StorageEncrypt
     :type queue_name: str
     :param credential:
         The credentials with which to authenticate. This is optional if the
-        account URL already has a SAS token. The value can be a SAS token string,
-        an instance of a AzureSasCredential or AzureNamedKeyCredential from azure.core.credentials,
-        an account shared access key, or an instance of a TokenCredentials class from azure.identity.
-        If the resource URI already contains a SAS token, this will be ignored in favor of an explicit credential
-        - except in the case of AzureSasCredential, where the conflicting SAS tokens will raise a ValueError.
-        If using an instance of AzureNamedKeyCredential, "name" should be the storage account name, and "key"
-        should be the storage account key.
+        account URL already has a SAS token. The value can be a SAS token string, an account
+        shared access key, or an instance of a TokenCredentials class from azure.identity.
     :keyword str api_version:
-        The Storage API version to use for requests. Default value is the most recent service version that is
-        compatible with the current SDK. Setting to an older version may result in reduced feature compatibility.
+        The Storage API version to use for requests. Default value is '2019-07-07'.
+        Setting to an older version may result in reduced feature compatibility.
     :keyword str secondary_hostname:
         The hostname of the secondary endpoint.
-    :keyword message_encode_policy: The encoding policy to use on outgoing messages.
+    :keyword encode_policy: The encoding policy to use on outgoing messages.
         Default is not to encode messages. Other options include :class:`TextBase64EncodePolicy`,
         :class:`BinaryBase64EncodePolicy` or `None`.
-    :keyword message_decode_policy: The decoding policy to use on incoming messages.
+    :keyword decode_policy: The decoding policy to use on incoming messages.
         Default value is not to decode messages. Other options include :class:`TextBase64DecodePolicy`,
         :class:`BinaryBase64DecodePolicy` or `None`.
 
@@ -87,7 +99,7 @@ class QueueClient(AsyncStorageAccountHostsMixin, QueueClientBase, StorageEncrypt
         self,
         account_url,  # type: str
         queue_name,  # type: str
-        credential=None,  # type: Optional[Union[str, Dict[str, str], AzureNamedKeyCredential, AzureSasCredential, "TokenCredential"]] # pylint: disable=line-too-long
+        credential=None,  # type: Optional[Any]
         **kwargs  # type: Any
     ):
         # type: (...) -> None
@@ -96,11 +108,9 @@ class QueueClient(AsyncStorageAccountHostsMixin, QueueClientBase, StorageEncrypt
         super(QueueClient, self).__init__(
             account_url, queue_name=queue_name, credential=credential, loop=loop, **kwargs
         )
-        self._client = AzureQueueStorage(self.url, base_url=self.url,
-                                         pipeline=self._pipeline, loop=loop)  # type: ignore
-        self._client._config.version = get_api_version(kwargs)  # pylint: disable=protected-access
+        self._client = AzureQueueStorage(self.url, pipeline=self._pipeline, loop=loop)  # type: ignore
+        self._client._config.version = kwargs.get('api_version', VERSION)  # pylint: disable=protected-access
         self._loop = loop
-        self._configure_encryption(kwargs)
 
     @distributed_trace_async
     async def create_queue(self, **kwargs):
@@ -137,7 +147,7 @@ class QueueClient(AsyncStorageAccountHostsMixin, QueueClientBase, StorageEncrypt
             return await self._client.queue.create(  # type: ignore
                 metadata=metadata, timeout=timeout, headers=headers, cls=deserialize_queue_creation, **kwargs
             )
-        except HttpResponseError as error:
+        except StorageErrorException as error:
             process_storage_error(error)
 
     @distributed_trace_async
@@ -169,7 +179,7 @@ class QueueClient(AsyncStorageAccountHostsMixin, QueueClientBase, StorageEncrypt
         timeout = kwargs.pop('timeout', None)
         try:
             await self._client.queue.delete(timeout=timeout, **kwargs)
-        except HttpResponseError as error:
+        except StorageErrorException as error:
             process_storage_error(error)
 
     @distributed_trace_async
@@ -198,7 +208,7 @@ class QueueClient(AsyncStorageAccountHostsMixin, QueueClientBase, StorageEncrypt
             response = await self._client.queue.get_properties(
                 timeout=timeout, cls=deserialize_queue_properties, **kwargs
             )
-        except HttpResponseError as error:
+        except StorageErrorException as error:
             process_storage_error(error)
         response.name = self.queue_name
         return response  # type: ignore
@@ -233,7 +243,7 @@ class QueueClient(AsyncStorageAccountHostsMixin, QueueClientBase, StorageEncrypt
             return await self._client.queue.set_metadata(  # type: ignore
                 timeout=timeout, headers=headers, cls=return_response_headers, **kwargs
             )
-        except HttpResponseError as error:
+        except StorageErrorException as error:
             process_storage_error(error)
 
     @distributed_trace_async
@@ -252,7 +262,7 @@ class QueueClient(AsyncStorageAccountHostsMixin, QueueClientBase, StorageEncrypt
             _, identifiers = await self._client.queue.get_access_policy(
                 timeout=timeout, cls=return_headers_and_deserialized, **kwargs
             )
-        except HttpResponseError as error:
+        except StorageErrorException as error:
             process_storage_error(error)
         return {s.id: s.access_policy or AccessPolicy() for s in identifiers}
 
@@ -305,7 +315,7 @@ class QueueClient(AsyncStorageAccountHostsMixin, QueueClientBase, StorageEncrypt
         signed_identifiers = identifiers  # type: ignore
         try:
             await self._client.queue.set_access_policy(queue_acl=signed_identifiers or None, timeout=timeout, **kwargs)
-        except HttpResponseError as error:
+        except StorageErrorException as error:
             process_storage_error(error)
 
     @distributed_trace_async
@@ -363,25 +373,13 @@ class QueueClient(AsyncStorageAccountHostsMixin, QueueClientBase, StorageEncrypt
         visibility_timeout = kwargs.pop('visibility_timeout', None)
         time_to_live = kwargs.pop('time_to_live', None)
         timeout = kwargs.pop('timeout', None)
-        try:
-            self._config.message_encode_policy.configure(
-                require_encryption=self.require_encryption,
-                key_encryption_key=self.key_encryption_key,
-                resolver=self.key_resolver_function,
-                encryption_version=self.encryption_version)
-        except TypeError:
-            warnings.warn(
-                "TypeError when calling message_encode_policy.configure. \
-                It is likely missing the encryption_version parameter. \
-                Consider updating your encryption information/implementation. \
-                Retrying without encryption_version."
-            )
-            self._config.message_encode_policy.configure(
-                require_encryption=self.require_encryption,
-                key_encryption_key=self.key_encryption_key,
-                resolver=self.key_resolver_function)
-        encoded_content = self._config.message_encode_policy(content)
-        new_message = GenQueueMessage(message_text=encoded_content)
+        self._config.message_encode_policy.configure(
+            require_encryption=self.require_encryption,
+            key_encryption_key=self.key_encryption_key,
+            resolver=self.key_resolver_function
+        )
+        content = self._config.message_encode_policy(content)
+        new_message = GenQueueMessage(message_text=content)
 
         try:
             enqueued = await self._client.messages.enqueue(
@@ -391,70 +389,14 @@ class QueueClient(AsyncStorageAccountHostsMixin, QueueClientBase, StorageEncrypt
                 timeout=timeout,
                 **kwargs
             )
-            queue_message = QueueMessage(content=content)
+            queue_message = QueueMessage(content=new_message.message_text)
             queue_message.id = enqueued[0].message_id
             queue_message.inserted_on = enqueued[0].insertion_time
             queue_message.expires_on = enqueued[0].expiration_time
             queue_message.pop_receipt = enqueued[0].pop_receipt
             queue_message.next_visible_on = enqueued[0].time_next_visible
             return queue_message
-        except HttpResponseError as error:
-            process_storage_error(error)
-
-    @distributed_trace_async
-    async def receive_message(self, **kwargs):
-        # type: (Optional[Any]) -> QueueMessage
-        """Removes one message from the front of the queue.
-
-        When the message is retrieved from the queue, the response includes the message
-        content and a pop_receipt value, which is required to delete the message.
-        The message is not automatically deleted from the queue, but after it has
-        been retrieved, it is not visible to other clients for the time interval
-        specified by the visibility_timeout parameter.
-
-        If the key-encryption-key or resolver field is set on the local service object, the message will be
-        decrypted before being returned.
-
-        :keyword int visibility_timeout:
-            If not specified, the default value is 0. Specifies the
-            new visibility timeout value, in seconds, relative to server time.
-            The value must be larger than or equal to 0, and cannot be
-            larger than 7 days. The visibility timeout of a message cannot be
-            set to a value later than the expiry time. visibility_timeout
-            should be set to a value smaller than the time-to-live value.
-        :keyword int timeout:
-            The server timeout, expressed in seconds.
-        :return:
-            Returns a message from the Queue.
-        :rtype: ~azure.storage.queue.QueueMessage
-
-        .. admonition:: Example:
-
-            .. literalinclude:: ../samples/queue_samples_message_async.py
-                :start-after: [START receive_one_message]
-                :end-before: [END receive_one_message]
-                :language: python
-                :dedent: 12
-                :caption: Receive one message from the queue.
-        """
-        visibility_timeout = kwargs.pop('visibility_timeout', None)
-        timeout = kwargs.pop('timeout', None)
-        self._config.message_decode_policy.configure(
-            require_encryption=self.require_encryption,
-            key_encryption_key=self.key_encryption_key,
-            resolver=self.key_resolver_function)
-        try:
-            message = await self._client.messages.dequeue(
-                number_of_messages=1,
-                visibilitytimeout=visibility_timeout,
-                timeout=timeout,
-                cls=self._config.message_decode_policy,
-                **kwargs
-            )
-            wrapped_message = QueueMessage._from_generated(  # pylint: disable=protected-access
-                message[0]) if message != [] else None
-            return wrapped_message
-        except HttpResponseError as error:
+        except StorageErrorException as error:
             process_storage_error(error)
 
     @distributed_trace
@@ -466,9 +408,7 @@ class QueueClient(AsyncStorageAccountHostsMixin, QueueClientBase, StorageEncrypt
         content and a pop_receipt value, which is required to delete the message.
         The message is not automatically deleted from the queue, but after it has
         been retrieved, it is not visible to other clients for the time interval
-        specified by the visibility_timeout parameter. The iterator will continuously
-        fetch messages until the queue is empty or max_messages is reached (if max_messages
-        is set).
+        specified by the visibility_timeout parameter.
 
         If the key-encryption-key or resolver field is set on the local service object, the messages will be
         decrypted before being returned.
@@ -478,19 +418,15 @@ class QueueClient(AsyncStorageAccountHostsMixin, QueueClientBase, StorageEncrypt
             messages to retrieve from the queue, up to a maximum of 32. If
             fewer are visible, the visible messages are returned. By default,
             a single message is retrieved from the queue with this operation.
-            `by_page()` can be used to provide a page iterator on the AsyncItemPaged if messages_per_page is set.
-            `next()` can be used to get the next page.
         :keyword int visibility_timeout:
-            If not specified, the default value is 30. Specifies the
+            If not specified, the default value is 0. Specifies the
             new visibility timeout value, in seconds, relative to server time.
-            The value must be larger than or equal to 1, and cannot be
+            The value must be larger than or equal to 0, and cannot be
             larger than 7 days. The visibility timeout of a message cannot be
             set to a value later than the expiry time. visibility_timeout
             should be set to a value smaller than the time-to-live value.
         :keyword int timeout:
             The server timeout, expressed in seconds.
-        :keyword int max_messages:
-            An integer that specifies the maximum number of messages to retrieve from the queue.
         :return:
             Returns a message iterator of dict-like Message objects.
         :rtype: ~azure.core.async_paging.AsyncItemPaged[~azure.storage.queue.QueueMessage]
@@ -507,7 +443,6 @@ class QueueClient(AsyncStorageAccountHostsMixin, QueueClientBase, StorageEncrypt
         messages_per_page = kwargs.pop('messages_per_page', None)
         visibility_timeout = kwargs.pop('visibility_timeout', None)
         timeout = kwargs.pop('timeout', None)
-        max_messages = kwargs.pop('max_messages', None)
         self._config.message_decode_policy.configure(
             require_encryption=self.require_encryption,
             key_encryption_key=self.key_encryption_key,
@@ -521,12 +456,8 @@ class QueueClient(AsyncStorageAccountHostsMixin, QueueClientBase, StorageEncrypt
                 cls=self._config.message_decode_policy,
                 **kwargs
             )
-            if max_messages is not None and messages_per_page is not None:
-                if max_messages < messages_per_page:
-                    raise ValueError("max_messages must be greater or equal to messages_per_page")
-            return AsyncItemPaged(command, results_per_page=messages_per_page,
-                                  page_iterator_class=MessagesPaged, max_messages=max_messages)
-        except HttpResponseError as error:
+            return AsyncItemPaged(command, results_per_page=messages_per_page, page_iterator_class=MessagesPaged)
+        except StorageErrorException as error:
             process_storage_error(error)
 
     @distributed_trace_async
@@ -604,27 +535,11 @@ class QueueClient(AsyncStorageAccountHostsMixin, QueueClientBase, StorageEncrypt
         if receipt is None:
             raise ValueError("pop_receipt must be present")
         if message_text is not None:
-            try:
-                self._config.message_encode_policy.configure(
-                    self.require_encryption,
-                    self.key_encryption_key,
-                    self.key_resolver_function,
-                    encryption_version=self.encryption_version
-                )
-            except TypeError:
-                warnings.warn(
-                    "TypeError when calling message_encode_policy.configure. \
-                    It is likely missing the encryption_version parameter. \
-                    Consider updating your encryption information/implementation. \
-                    Retrying without encryption_version."
-                )
-                self._config.message_encode_policy.configure(
-                    self.require_encryption,
-                    self.key_encryption_key,
-                    self.key_resolver_function
-                )
-            encoded_message_text = self._config.message_encode_policy(message_text)
-            updated = GenQueueMessage(message_text=encoded_message_text)
+            self._config.message_encode_policy.configure(
+                self.require_encryption, self.key_encryption_key, self.key_resolver_function
+            )
+            message_text = self._config.message_encode_policy(message_text)
+            updated = GenQueueMessage(message_text=message_text)
         else:
             updated = None  # type: ignore
         try:
@@ -645,7 +560,7 @@ class QueueClient(AsyncStorageAccountHostsMixin, QueueClientBase, StorageEncrypt
             new_message.pop_receipt = response["popreceipt"]
             new_message.next_visible_on = response["time_next_visible"]
             return new_message
-        except HttpResponseError as error:
+        except StorageErrorException as error:
             process_storage_error(error)
 
     @distributed_trace_async
@@ -702,7 +617,7 @@ class QueueClient(AsyncStorageAccountHostsMixin, QueueClientBase, StorageEncrypt
             for peeked in messages:
                 wrapped_messages.append(QueueMessage._from_generated(peeked))  # pylint: disable=protected-access
             return wrapped_messages
-        except HttpResponseError as error:
+        except StorageErrorException as error:
             process_storage_error(error)
 
     @distributed_trace_async
@@ -725,7 +640,7 @@ class QueueClient(AsyncStorageAccountHostsMixin, QueueClientBase, StorageEncrypt
         timeout = kwargs.pop('timeout', None)
         try:
             await self._client.messages.clear(timeout=timeout, **kwargs)
-        except HttpResponseError as error:
+        except StorageErrorException as error:
             process_storage_error(error)
 
     @distributed_trace_async
@@ -775,5 +690,5 @@ class QueueClient(AsyncStorageAccountHostsMixin, QueueClientBase, StorageEncrypt
             await self._client.message_id.delete(
                 pop_receipt=receipt, timeout=timeout, queue_message_id=message_id, **kwargs
             )
-        except HttpResponseError as error:
+        except StorageErrorException as error:
             process_storage_error(error)
